/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */

import { Client } from 'discord.js';
import { ClientEvents } from 'discord.js';
import { ClientOptions } from 'discord.js';
import { ClientPlugin } from '@cordjs/client';
import { Context } from '@cordjs/client';
import { Middleware } from '@cordjs/client';
import { MiddlewareGroup } from '@cordjs/client';
import { PluginOptions } from '@cordjs/client';

/**
 * Gateway plugin
 *
 * @public
 */
declare class Gateway extends ClientPlugin<GatewayMiddleware, GatewayOptions> {
    id: string;
    discordClient?: Client;
    constructor(options: GatewayOptions);
    start(): Promise<void>;
}
export { Gateway }
export default Gateway;

/**
 * The context
 */
export declare class GatewayContext<K extends keyof ClientEvents> extends Context {
    data: ClientEvents[K];
    constructor(path: string[], data: ClientEvents[K]);
}

/**
 * {@link Gateway} middleware
 */
export declare type GatewayMiddleware = {
    gateway: MiddlewareGroup<{
        [K in keyof ClientEvents]: Middleware<GatewayContext<K>>;
    }>;
};

/**
 * {@link Gateway} options
 * @public
 */
export declare interface GatewayOptions extends PluginOptions<GatewayMiddleware> {
    /**
     * The token to login with
     */
    token: string;
    /**
     * The {@link https://discord.js.org/#/docs/main/stable/typedef/ClientOptions | Discord.js client options}
     */
    client: ClientOptions;
    /**
     * Whether or not to catch all events
     *
     * @remarks
     * This will monkey patch the `emit` function
     *
     * @defaultValue false
     */
    catchAll?: boolean;
}

export { }
