/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */
import { ClientPlugin, Middleware, MiddlewareGroup, Context, PluginOptions } from '@cordjs/client';
import { Client, ClientEvents, ClientOptions } from 'discord.js';
/**
 * {@link Gateway} options
 * @public
 */
export interface GatewayOptions extends PluginOptions<GatewayMiddleware> {
    /**
     * The token to login with
     */
    token: string;
    /**
     * The {@link https://discord.js.org/#/docs/main/stable/typedef/ClientOptions | Discord.js client options}
     */
    client: ClientOptions;
    /**
     * Whether or not to catch all events
     *
     * @remarks
     * This will monkey patch the `emit` function
     *
     * @defaultValue false
     */
    catchAll?: boolean;
}
/**
 * {@link Gateway} middleware
 */
export declare type GatewayMiddleware = {
    gateway: MiddlewareGroup<{
        [K in keyof ClientEvents]: Middleware<GatewayContext<K>>;
    }>;
};
/**
 * Gateway plugin
 *
 * @public
 */
export declare class Gateway extends ClientPlugin<GatewayMiddleware, GatewayOptions> {
    id: string;
    discordClient?: Client;
    constructor(options: GatewayOptions);
    start(): Promise<void>;
}
/**
 * The context
 */
export declare class GatewayContext<K extends keyof ClientEvents> extends Context {
    data: ClientEvents[K];
    constructor(path: string[], data: ClientEvents[K]);
}
export default Gateway;
//# sourceMappingURL=index.d.ts.map