"use strict";
/**
 * Connect to the gateway with Cord.js
 * @packageDocumentation
 *
 * @remarks
 * This plugin uses {@link https://discord.js.org/ | Discord.js} under the hood.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GatewayContext = exports.Gateway = void 0;
const client_1 = require("@cordjs/client");
const discord_js_1 = require("discord.js");
/**
 * Gateway plugin
 *
 * @public
 */
class Gateway extends client_1.ClientPlugin {
    constructor(options) {
        super(options, ['gateway']);
        this.id = '@cordjs/gateway';
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const client = new discord_js_1.Client(this.options.client);
            if (this.options.catchAll) {
                const emit = client.emit;
                client.emit = (eventName, ...args) => {
                    this.pluginRunMiddleware(new GatewayContext([this.middleware.gateway, eventName], args));
                    return emit.apply(client, [eventName, args]);
                };
            }
            else {
                const added = {};
                for (const middleware of this.client.middleware) {
                    if (middleware.path.length === 2 &&
                        middleware.path[0] === this.middleware.gateway) {
                        if (!(middleware.path[1] in added)) {
                            const eventName = middleware.path[1];
                            added[eventName] = true;
                            client.on(eventName, (...args) => {
                                this.pluginRunMiddleware(new GatewayContext([this.middleware.gateway, eventName], args));
                            });
                        }
                    }
                }
            }
            this.discordClient = client;
            yield client.login(this.options.token);
        });
    }
}
exports.Gateway = Gateway;
/**
 * The context
 */
class GatewayContext extends client_1.Context {
    constructor(path, data) {
        super(path);
        this.data = data;
    }
}
exports.GatewayContext = GatewayContext;
exports.default = Gateway;
//# sourceMappingURL=index.js.map